#include "aduc7061.h"

#define IRQ3CLRI_BIT           19
#define IRQ2CLRI_BIT           18
#define IRQ1CLRI_BIT           14
#define IRQ0CLRI_BIT           13

/* IRQCONN Bit settings */
#define IRQCONN_IRQ_NEST_BIT   0
#define IRQCONN_FIQ_NEST_BIT   1

/* IRQP0 MMR Bit Designations */
#define IRQP0_T3PI_BIT         24
#define IRQP0_T2PI_BIT         20
#define IRQP0_T1PI_BIT         16
#define IRQP0_T0PI_BIT         12
#define IRQP0_SWIINTP_BIT      4

/* IRQP1 MMR Bit Designations */
#define IRQP1_I2CMPI_BIT       28
#define IRQP1_IRQ1PI_BIT       24
#define IRQP1_IRQ0PI_BIT       20
#define IRQP1_SPIMPI_BIT       16
#define IRQP1_UARTPI_BIT       12
#define IRQP1_ADCPI_BIT        8


/* IRQP2 MMR Bit Designations */
#define IRQP2_IRQ3PI_BIT       12
#define IRQP2_IRQ2PI_BIT       8
#define IRQP2_PWMPI_BIT        4
#define IRQP2_I2CSPI_BIT       0


typedef void (*tyVctHndlr)(void);

void unused_isr(void)
{
  while(1);
}

void IRQ0_FIQ_SOURCE(void) __attribute__((weak,alias("unused_isr")));
void IRQ1_SW_INT(void) __attribute__((weak,alias("unused_isr")));
void IRQ3_T0_INT(void) __attribute__((weak,alias("unused_isr")));
void IRQ4_T1_INT(void) __attribute__((weak,alias("unused_isr")));
void IRQ5_T2_INT(void) __attribute__((weak,alias("unused_isr")));
void IRQ6_T3_INT(void) __attribute__((weak,alias("unused_isr")));
void IRQ10_ADC_INT(void) __attribute__((weak,alias("unused_isr")));
void IRQ11_UART_INT(void) __attribute__((weak,alias("unused_isr")));
void IRQ12_SPIM_INT(void) __attribute__((weak,alias("unused_isr")));
void IRQ13_XIRQ0_INT(void) __attribute__((weak,alias("unused_isr")));
void IRQ14_XIRQ1_INT(void) __attribute__((weak,alias("unused_isr")));
void IRQ15_I2CM_INT(void) __attribute__((weak,alias("unused_isr")));
void IRQ16_I2CS_INT(void) __attribute__((weak,alias("unused_isr")));
void IRQ17_PWM_INT(void) __attribute__((weak,alias("unused_isr")));
void IRQ18_XIRQ2_INT(void) __attribute__((weak,alias("unused_isr")));
void IRQ19_XIRQ3_INT(void) __attribute__((weak,alias("unused_isr")));
void IRQx_NULL_INT(void) __attribute__((weak,alias("unused_isr")));

__attribute__ ((section(".vectors"), used))
tyVctHndlr _VectorsTable[32] __attribute__ ((aligned (128)))  =
{
  IRQ0_FIQ_SOURCE,             /* 0  - Any FIQ active       */
  IRQ1_SW_INT    ,             /* 1  - Software             */
  IRQx_NULL_INT  ,             /* 2  - Unused */
  IRQ3_T0_INT    ,             /* 3  - Timer 0              */
  IRQ4_T1_INT    ,             /* 4  - Timer 1              */
  IRQ5_T2_INT    ,             /* 5  - Timer 2              */
  IRQ6_T3_INT    ,             /* 6  - Timer 3              */
  IRQx_NULL_INT  ,             /* 7  - Unused */
  IRQx_NULL_INT  ,             /* 8  - Unused */
  IRQx_NULL_INT  ,             /* 9  - Unused */
  IRQ10_ADC_INT  ,             /* 10 - ADC Interrupt        */
  IRQ11_UART_INT ,             /* 11 - UART interrupt       */
  IRQ12_SPIM_INT  ,            /* 12 - SPI Master interrupt */
  IRQ13_XIRQ0_INT,             /* 13 - External interrupt 0 */
  IRQ14_XIRQ1_INT,             /* 14 - External interrupt 1 */
  IRQ15_I2CM_INT ,             /* 15 - I2C Master interrupt */
  IRQ16_I2CS_INT ,             /* 16 - I2C Slave interrupt  */
  IRQ17_PWM_INT ,              /* 17 - PWM trip interrupt   */
  IRQ18_XIRQ2_INT,             /* 18 - External interrupt 2 */
  IRQ19_XIRQ3_INT,             /* 19 - External interrupt 3 */
  IRQx_NULL_INT  ,             /* 20 - Unused */
  IRQx_NULL_INT  ,             /* 21 - Unused */
  IRQx_NULL_INT  ,             /* 22 - Unused */
  IRQx_NULL_INT  ,             /* 23 - Unused */
  IRQx_NULL_INT  ,             /* 24 - Unused */
  IRQx_NULL_INT  ,             /* 25 - Unused */
  IRQx_NULL_INT  ,             /* 26 - Unused */
  IRQx_NULL_INT  ,             /* 27 - Unused */
  IRQx_NULL_INT  ,             /* 28 - Unused */
  IRQx_NULL_INT  ,             /* 29 - Unused */
  IRQx_NULL_INT  ,             /* 30 - Unused */
  IRQx_NULL_INT  ,             /* 31 - Unused */
};

extern int main(void);

__attribute__ ((section(".startup")))
void start(void)
{
	/************************* CPU */
	POWKEY1 = 0x1;									/* NAP Mode before writing POWCON0 */
	POWCON0 = 0x78;									/* XPD | PLLPD | PPD | COREPD and Set core to max CPU speed of 10.24Mhz */
	POWKEY2 = 0xF4;									/* Just after writing POWCON0 */

		/* Let the VIC know about our vector table */
   IRQBASE = (unsigned long)(((unsigned long) _VectorsTable) >> 7);
		/* Enable nesting and prioritisation of IRQ and FIQ interrupts */
   IRQCONN = (0x1 << IRQCONN_IRQ_NEST_BIT) | (0x1 << IRQCONN_FIQ_NEST_BIT);
		/* Set priorities (0x0 is the highest, 0x7 the lowest) */

   IRQP0 = (0x0 << IRQP0_T3PI_BIT) | (0x0 << IRQP0_T2PI_BIT) |
           (0x0 << IRQP0_T1PI_BIT) | (0x0 << IRQP0_T0PI_BIT) |
           (0x0 << IRQP0_SWIINTP_BIT);

   IRQP1 = (0x0 << IRQP1_I2CMPI_BIT) | (0x0 << IRQP1_IRQ1PI_BIT) |
           (0x0 << IRQP1_IRQ0PI_BIT) | (0x0 << IRQP1_SPIMPI_BIT) |
           (0x0 << IRQP1_UARTPI_BIT) | (0x0 << IRQP1_ADCPI_BIT );

   IRQP2 = (0x0 << IRQP2_IRQ3PI_BIT) | (0x0 << IRQP2_IRQ2PI_BIT) |
           (0x0 << IRQP2_PWMPI_BIT)  | (0x0 << IRQP2_I2CSPI_BIT);


  main();
  while(1);
}

/*    Function Pointers for Interrupts  */
typedef void (* tyVctHndlr) (void);

tyVctHndlr    IRQ     = (tyVctHndlr)0x0;
tyVctHndlr    SWI     = (tyVctHndlr)0x0;
tyVctHndlr    FIQ     = (tyVctHndlr)0x0;
tyVctHndlr    UNDEF   = (tyVctHndlr)0x0;
tyVctHndlr    PABORT  = (tyVctHndlr)0x0;
tyVctHndlr    DABORT  = (tyVctHndlr)0x0;

void	ADI_IRQ_Interrupt_Setup(void) __attribute__ ((interrupt ("IRQ")));
void	ADI_SWI_Interrupt_Setup(void) __attribute__ ((interrupt ("SWI")));
void	ADI_FIQ_Interrupt_Setup(void) __attribute__ ((interrupt ("FIQ")));
void	ADI_UNDEF_Interrupt_Setup(void) __attribute__ ((interrupt ("UNDEF")));
void	ADI_PABORT_Interrupt_Setup(void) __attribute__ ((interrupt ("ABORT")));
void	ADI_DABORT_Interrupt_Setup(void)  __attribute__ ((interrupt ("ABORT")));

void	ADI_IRQ_Interrupt_Setup(void)
{
	if ( *IRQ !=0x00)
	{
		IRQ();
	}
}

void	ADI_FIQ_Interrupt_Setup(void)
{
	if ( *FIQ !=0x00)
	{
		FIQ();
	}
}

void	ADI_SWI_Interrupt_Setup(void)
{
	if ( *SWI !=0x00)
	{
		SWI();
	}
}

void	ADI_UNDEF_Interrupt_Setup(void)
{
	if ( *UNDEF !=0x00)
	{
		UNDEF();
	}
}

void	ADI_PABORT_Interrupt_Setup(void)
{
	if ( *PABORT !=0x00)
	{
		PABORT();
	}
}

void	ADI_DABORT_Interrupt_Setup(void)
{
	if ( *DABORT !=0x00)
	{
		DABORT();
	}
}
