#include <stdint.h>

/* INTERRUPT CONTROLLER */
#define INTBASE        (*(volatile uint32_t*) 0xFFFF0000)
#define IRQSTA         (*(volatile uint32_t*) 0xFFFF0000)
#define IRQSIG         (*(volatile uint32_t*) 0xFFFF0004)
#define IRQEN          (*(volatile uint32_t*) 0xFFFF0008)
#define IRQCLR         (*(volatile uint32_t*) 0xFFFF000C)
#define SWICFG         (*(volatile uint32_t*) 0xFFFF0010)
#define IRQBASE        (*(volatile uint32_t*) 0xFFFF0014)
#define IRQVEC         (*(volatile uint32_t*) 0xFFFF001C)
#define IRQP0          (*(volatile uint32_t*) 0xFFFF0020)
#define IRQP1          (*(volatile uint32_t*) 0xFFFF0024)
#define IRQP2          (*(volatile uint32_t*) 0xFFFF0028)
#define IRQCONN        (*(volatile uint32_t*) 0xFFFF0030)
#define IRQCONE        (*(volatile uint32_t*) 0xFFFF0034)
#define IRQCLRE        (*(volatile uint32_t*) 0xFFFF0038)
#define IRQSTAN        (*(volatile uint32_t*) 0xFFFF003C)
#define FIQSTA         (*(volatile uint32_t*) 0xFFFF0100)
#define FIQSIG         (*(volatile uint32_t*) 0xFFFF0104)
#define FIQEN          (*(volatile uint32_t*) 0xFFFF0108)
#define FIQCLR         (*(volatile uint32_t*) 0xFFFF010C)
#define FIQVEC         (*(volatile uint32_t*) 0xFFFF011C)
#define FIQSTAN        (*(volatile uint32_t*) 0xFFFF013C)

/* REMAP AND SYSTEM CONTROL */
#define REMAPBASE      (*(volatile uint32_t*) 0xFFFF0200)
// =====
#define REMAP          (*(volatile uint32_t*) 0xFFFF0220)
// =====
#define RSTSTA         (*(volatile uint32_t*) 0xFFFF0230)
#define RSTCLR         (*(volatile uint32_t*) 0xFFFF0234)
#define RST_EXT        (1<<3)
#define RST_SOFT       (1<<2)
#define RST_WDOG       (1<<1)
#define RST_POR        (1<<0)

/* TIMER 0 */
#define T0BASE         (*(volatile uint32_t*) 0xFFFF0320)
// =====
#define T0LD           (*(volatile uint32_t*) 0xFFFF0320)
#define T0VAL          (*(volatile uint32_t*) 0xFFFF0324)
// =====
#define T0CON          (*(volatile uint32_t*) 0xFFFF0328)
#define T0PVAL(n)      ((n&0xFF)<<24)
#define T0PEN          (1<<23)
#define T0PCF          (1<<19)
#define T0SRCI         (1<<18)
#define T0CAPEN        (1<<17)
#define T0CAPSEL(n)    ((n&0x3F)<<12)
#define T0CLKSEL(n)    ((n&0x3)<<9)
#define T0DIR          (1<<8)
#define T0EN           (1<<7)
#define T0MOD          (1<<6)
#define T0FORMAT(n)    ((n&0x3)<<4)
#define T0SCALE(n)     ((n&0xF)<<0)
// =====
#define T0CLRI         (*(volatile uint32_t*) 0xFFFF032C)
#define T0CAP          (*(volatile uint32_t*) 0xFFFF0330)

/* WAKE UP TIMER */
#define T1BASE         (*(volatile uint32_t*) 0xFFFF0340)
// =====
#define T1LD           (*(volatile uint32_t*) 0xFFFF0340)
#define T1VAL          (*(volatile uint32_t*) 0xFFFF0344)
// =====
#define T1CON          (*(volatile uint32_t*) 0xFFFF0348)
#define T1CLKSEL(n)    ((n&0x3)<<9)
#define T1DIR          (1<<8)
#define T1EN           (1<<7)
#define T1MOD          (1<<6)
#define T1FORMAT(n)    ((n&0x3)<<4)
#define T1SCALE(n)     ((n&0xF)<<0)
// =====
#define T1CLRI         (*(volatile uint32_t*) 0xFFFF034C)

/* WATCHDOG TIMER */
#define T2BASE         (*(volatile uint32_t*) 0xFFFF0360)
// =====
#define T2LD           (*(volatile uint32_t*) 0xFFFF0360)
#define T2VAL          (*(volatile uint32_t*) 0xFFFF0364)
// =====
#define T2CON          (*(volatile uint32_t*) 0xFFFF0368)
#define T2DIR          (1<<8)
#define T2EN           (1<<7)
#define T2MOD          (1<<6)
#define WDOGMDEN        (1<<5)
#define T2SCALE(n)     ((n&0x3)<<2)
#define WDOGENI        (1<<1)
#define T2PDOFF        (1<<0)
// =====
#define T2CLRI         (*(volatile uint32_t*) 0xFFFF036C)

/* TIMER 3 */
#define T3BASE         (*(volatile uint32_t*) 0xFFFF0380)
// =====
#define T3LD           (*(volatile uint32_t*) 0xFFFF0380)
#define T3VAL          (*(volatile uint32_t*) 0xFFFF0384)
// =====
#define T3CON          (*(volatile uint32_t*) 0xFFFF0388)
#define T3CAPEN        (1<<17)
#define T3CAPSEL(n)    ((n&0x3F)<<12)
#define T3CLKSEL(n)    ((n&0x3)<<9)
#define T3DIR          (1<<8)
#define T3EN           (1<<7)
#define T3MOD          (1<<6)
#define T3SCALE(n)     ((n&0xF)<<0)
// =====
#define T3CLRI         (*(volatile uint32_t*) 0xFFFF038C)
#define T3CAP          (*(volatile uint32_t*) 0xFFFF0390)

/* PLL AND OSCILLATOR CONTROL */
#define PLLBASE        (*(volatile uint32_t*) 0xFFFF0400)
#define POWKEY1        (*(volatile uint32_t*) 0xFFFF0404)
#define POWCON0        (*(volatile uint32_t*) 0xFFFF0408)
#define POWKEY2        (*(volatile uint32_t*) 0xFFFF040C)
#define PLLKEY1        (*(volatile uint32_t*) 0xFFFF0410)
#define PLLCON         (*(volatile uint32_t*) 0xFFFF0414)
#define PLLKEY2        (*(volatile uint32_t*) 0xFFFF0418)
#define GP0KEY1        (*(volatile uint32_t*) 0xFFFF0464)
#define GP0CON1        (*(volatile uint32_t*) 0xFFFF0468)
#define GP0KEY2        (*(volatile uint32_t*) 0xFFFF046C)

/* ADC INTERFACE REGISTERS */
#define ADCBASE        (*(volatile uint32_t*) 0xFFFF0500)
// =====
#define ADCSTA         (*(volatile uint32_t*) 0xFFFF0500)
#define ADCCALSTA      (1<<15)
#define ADC1CERR       (1<<13)
#define ADC0CERR       (1<<12)
#define ADC0ATHEX      (1<<6)
#define ADC0THEX       (1<<4)
#define ADC0OVR        (1<<3)
#define ADC1RDY        (1<<1)
#define ADC0RDY        (1<<0)
// =====
#define ADCMSKI        (*(volatile uint32_t*) 0xFFFF0504)
#define ADC0ATHEX_INTEN (1<<6)
#define ADC0THEX_INTEN (1<<4)
#define ADC0OVR_INTEN  (1<<3)
#define ADC1RDY_INTEN  (1<<1)
#define ADC0RDY_INTEN  (1<<0)
// =====
#define ADCMDE         (*(volatile uint32_t*) 0xFFFF0508)
#define ADCCLKSEL      (1<<7)
#define ADCLPMEN       (1<<5)
#define ADCLPMCFG(n)   ((n&0x3)<<3)
#define ADCMD(n)       ((n&0x7)<<0)
// =====
#define ADC0CON        (*(volatile uint32_t*) 0xFFFF050C)
#define ADC0EN         (1<<15)
#define ADC0DIAG(n)    ((n&0x3)<<13)
#define HIGHEXTREF0    (1<<12)
#define AMP_CM         (1<<11)
#define ADC0CODE       (1<<10)
#define ADC0CH(n)      ((n&0xF)<<6)
#define ADC0REF(n)     ((n&0x3)<<4)
#define ADC0PGA(n)     ((n&0xF)<<0)
// =====
#define ADC1CON        (*(volatile uint32_t*) 0xFFFF0510)
#define ADC1EN         (1<<15)
#define ADC1DIAG(n)    ((n&0x3)<<13)
#define HIGHEXTREF1    (1<<12)
#define ADC1CODE       (1<<11)
#define ADC1CH(n)      ((n&0xF)<<7)
#define ADC1REF(n)     ((n&0x7)<<4)
#define BUF_BYPASS(n)  ((n&0x3)<<2)
#define ADC1PGA(n)     ((n&0x3)<<0)
// =====
#define ADCFLT         (*(volatile uint32_t*) 0xFFFF0514)
#define CHOPEN         (1<<15)
#define RAVG2          (1<<14)
#define AF(n)          ((n&0x3F)<<8)
#define NOTCH2         (1<<7)
#define SF(n)          ((n&0x7F)<<0)
// =====
#define ADCCFG         (*(volatile uint32_t*) 0xFFFF0518)
#define GNDSW_EN       (1<<7)
#define ADC0ACCEN(n)   ((n&0x3)<<5)
#define ADC0CMPEN(n)   ((n&0x3)<<3)
#define ADC0OREN       (1<<2)
#define GNDSW_RES_EN   (1<<1)
#define ADCRCEN        (1<<0)
// =====
#define ADC0DAT        (*(volatile uint32_t*) 0xFFFF051C)
#define ADC1DAT        (*(volatile uint32_t*) 0xFFFF0520)
#define ADC0OF         (*(volatile uint32_t*) 0xFFFF0524)
#define ADC1OF         (*(volatile uint32_t*) 0xFFFF0528)
#define ADC0GN         (*(volatile uint32_t*) 0xFFFF052C)
#define ADC1GN         (*(volatile uint32_t*) 0xFFFF0530)
#define ADCORCR        (*(volatile uint32_t*) 0xFFFF0534)
#define ADCORCV        (*(volatile uint32_t*) 0xFFFF0538)
#define ADCOTH         (*(volatile uint32_t*) 0xFFFF053C)
#define ADCOTHC        (*(volatile uint32_t*) 0xFFFF0540)
#define ADCOTHV        (*(volatile uint32_t*) 0xFFFF0544)
#define ADCOACC        (*(volatile uint32_t*) 0xFFFF0548)
#define ADCOATH        (*(volatile uint32_t*) 0xFFFF054C)
// =====
#define IEXCON         (*(volatile uint32_t*) 0xFFFF0570)
#define IEXC1_EN       (1<<7)
#define IEXC0_EN       (1<<6)
#define IEXC1_DIR      (1<<5)
#define IEXC0_DIR      (1<<4)
#define IOUT(n)        ((n&0x7)<<1)
#define IOUT0          (1<<0)

/* DAC INTERFACE REGISTERS */
#define DACBASE        (*(volatile uint32_t*) 0xFFFF0600)
// =====
#define DACCON         (*(volatile uint32_t*) 0xFFFF0600)
#define DACPD          (1<<9)
#define DACBUFLP       (1<<8)
#define OPAMP          (1<<7)
#define DACBUFBYPASS   (1<<6)
#define DACCLK         (1<<5)
#define DACCLRn        (1<<4)
#define DACMODE        (1<<3)
#define DACRATE           (1<<2)
#define DACRANGE(n)    ((n&0x3)<<0)
// =====
#define DACDAT         (*(volatile uint32_t*) 0xFFFF0604)

/* 450 COMPATIABLE UART CORE REGISTERS */
#define UARTBASE       (*(volatile uint32_t*) 0xFFFF0700)
#define COMTX          (*(volatile uint32_t*) 0xFFFF0700)
#define COMRX          (*(volatile uint32_t*) 0xFFFF0700)
#define COMDIV0        (*(volatile uint32_t*) 0xFFFF0700)
#define COMIEN0        (*(volatile uint32_t*) 0xFFFF0704)
#define COMDIV1        (*(volatile uint32_t*) 0xFFFF0704)
#define COMIID0        (*(volatile uint32_t*) 0xFFFF0708)
#define COMCON0        (*(volatile uint32_t*) 0xFFFF070C)
#define COMCON1        (*(volatile uint32_t*) 0xFFFF0710)
#define COMSTA0        (*(volatile uint32_t*) 0xFFFF0714)
#define COMSTA1        (*(volatile uint32_t*) 0xFFFF0718)
#define COMSCR         (*(volatile uint32_t*) 0xFFFF071C)
#define COMDIV2        (*(volatile uint32_t*) 0xFFFF072C)

/* I2C BUS PERIPHERAL DEVICE  */
#define I2CBASE        (*(volatile uint32_t*) 0xFFFF0900)
#define I2CMCON        (*(volatile uint32_t*) 0xFFFF0900)
#define I2CMSTA        (*(volatile uint32_t*) 0xFFFF0904)
#define I2CMRX         (*(volatile uint32_t*) 0xFFFF0908)
#define I2CMTX         (*(volatile uint32_t*) 0xFFFF090C)
#define I2CMCNT0       (*(volatile uint32_t*) 0xFFFF0910)
#define I2CMCNT1       (*(volatile uint32_t*) 0xFFFF0914)
#define I2CADR0        (*(volatile uint32_t*) 0xFFFF0918)
#define I2CADR1        (*(volatile uint32_t*) 0xFFFF091C)
#define I2CDIV         (*(volatile uint32_t*) 0xFFFF0924)
#define I2CSCON        (*(volatile uint32_t*) 0xFFFF0928)
#define I2CSSTA        (*(volatile uint32_t*) 0xFFFF092C)
#define I2CSRX         (*(volatile uint32_t*) 0xFFFF0930)
#define I2CSTX         (*(volatile uint32_t*) 0xFFFF0934)
#define I2CALT         (*(volatile uint32_t*) 0xFFFF0938)
#define I2CID0         (*(volatile uint32_t*) 0xFFFF093C)
#define I2CID1         (*(volatile uint32_t*) 0xFFFF0940)
#define I2CID2         (*(volatile uint32_t*) 0xFFFF0944)
#define I2CID3         (*(volatile uint32_t*) 0xFFFF0948)
#define I2CFSTA        (*(volatile uint32_t*) 0xFFFF094C)

/* SERIAL PORT INTERFACE PERIPHERAL */
#define SPIBASE        (*(volatile uint32_t*) 0xFFFF0A00)
#define SPISTA         (*(volatile uint32_t*) 0xFFFF0A00)
#define SPIRX          (*(volatile uint32_t*) 0xFFFF0A04)
#define SPITX          (*(volatile uint32_t*) 0xFFFF0A08)
#define SPIDIV         (*(volatile uint32_t*) 0xFFFF0A0C)
#define SPICON         (*(volatile uint32_t*) 0xFFFF0A10)

/* GPIO AND SERIAL PORT MUX */
#define GPIOBASE       (*(volatile uint32_t*) 0xFFFF0D00)
// =====
#define GP0CON0        (*(volatile uint32_t*) 0xFFFF0D00)
#define GP1CON         (*(volatile uint32_t*) 0xFFFF0D04)
#define GP2CON         (*(volatile uint32_t*) 0xFFFF0D08)
#define GPxF(n,f)      ((f&0x3)<<(4*n))
// =====
#define GP0DAT         (*(volatile uint32_t*) 0xFFFF0D20)
#define GP0SET         (*(volatile uint32_t*) 0xFFFF0D24)
#define GP0CLR         (*(volatile uint32_t*) 0xFFFF0D28)
#define GPxDIR(n)      ((n&0xFF)<<24)
#define GPxOUT(n)      ((n&0xFF)<<16)
#define GPxIN(n)       ((n&0xFF)<<0)
// =====
#define GP0PAR         (*(volatile uint32_t*) 0xFFFF0D2C)
#define GPxGPL(n)      ((n&0xFF)<<16)
#define GPxGPDS(n)     ((n&0xFF)<<8)
#define GPxGPPD(n)     ((n&0xFF)<<0)
// =====
#define GP1DAT         (*(volatile uint32_t*) 0xFFFF0D30)
#define GP1SET         (*(volatile uint32_t*) 0xFFFF0D34)
#define GP1CLR         (*(volatile uint32_t*) 0xFFFF0D38)
#define GP1PAR         (*(volatile uint32_t*) 0xFFFF0D3C)
#define GP2DAT         (*(volatile uint32_t*) 0xFFFF0D40)
#define GP2SET         (*(volatile uint32_t*) 0xFFFF0D44)
#define GP2CLR         (*(volatile uint32_t*) 0xFFFF0D48)
#define GP2PAR         (*(volatile uint32_t*) 0xFFFF0D4C)

/* FLASH CONTROL INTERFACE */
#define FLASHBASE      (*(volatile uint32_t*) 0xFFFF0E00)
// =====
#define FEESTA         (*(volatile uint32_t*) 0xFFFF0E00)
#define FEEINT         (1<<3)
#define FEEBUSY        (1<<2)
#define FEEFAIL        (1<<1)
#define FEEPASS        (1<<0)
// =====
#define FEEMOD         (*(volatile uint32_t*) 0xFFFF0E04)
#define FEEMODBIT      (1<<8)
#define FEEMODKEY(n)   ((n&0x7)<<5)
#define FEEINTEN       (1<<4)
#define FEEWEN     (1<<3)
// =====
#define FEECON         (*(volatile uint32_t*) 0xFFFF0E08)
// =====
#define FEEDAT         (*(volatile uint32_t*) 0xFFFF0E0C)
#define FEEADR         (*(volatile uint32_t*) 0xFFFF0E10)
#define FEESIG         (*(volatile uint32_t*) 0xFFFF0E18)
#define FEEPRO         (*(volatile uint32_t*) 0xFFFF0E1C)
#define FEEHID         (*(volatile uint32_t*) 0xFFFF0E20)

/* PWM */
#define PWMBASE        (*(volatile uint32_t*) 0xFFFF0F80)
// =====
#define PWMCON         (*(volatile uint32_t*) 0xFFFF0F80)
#define SYNC           (1<<14)
#define PWM5INV        (1<<13)
#define PWM3INV        (1<<12)
#define PWM1INV        (1<<11)
#define PWMTRIP        (1<<10)
#define ENA            (1<<9)
#define PWMCP(n)       ((n&0x7)<<6)
#define POINV          (1<<5)
#define HOFF           (1<<4)
#define LCOMP          (1<<3)
#define DIR            (1<<2)
#define HMODE          (1<<1)
#define PWMEN          (1<<0)
// =====
#define PWM0COM0       (*(volatile uint32_t*) 0xFFFF0F84)
#define PWM0COM1       (*(volatile uint32_t*) 0xFFFF0F88)
#define PWM0COM2       (*(volatile uint32_t*) 0xFFFF0F8C)
#define PWM0LEN        (*(volatile uint32_t*) 0xFFFF0F90)
#define PWM1COM0       (*(volatile uint32_t*) 0xFFFF0F94)
#define PWM1COM1       (*(volatile uint32_t*) 0xFFFF0F98)
#define PWM1COM2       (*(volatile uint32_t*) 0xFFFF0F9C)
#define PWM1LEN        (*(volatile uint32_t*) 0xFFFF0FA0)
#define PWM2COM0       (*(volatile uint32_t*) 0xFFFF0FA4)
#define PWM2COM1       (*(volatile uint32_t*) 0xFFFF0FA8)
#define PWM2COM2       (*(volatile uint32_t*) 0xFFFF0FAC)
#define PWM2LEN        (*(volatile uint32_t*) 0xFFFF0FB0)
#define PWMCLRI        (*(volatile uint32_t*) 0xFFFF0FB8)

/*    Function Pointers for Interrupts  */
typedef void (* tyVctHndlr) (void);

extern tyVctHndlr    IRQ;
extern tyVctHndlr    SWI;
extern tyVctHndlr    FIQ;
extern tyVctHndlr    UNDEF;
extern tyVctHndlr    PABORT;
extern tyVctHndlr    DABORT;

#define SW_INT                 1
#define T0_INT                 3
#define T1_INT                 4
#define T2_INT                 5
#define T3_INT                 6
#define ADC_INT                10
#define UART_INT               11
#define SPIM_INT               12
#define XIRQ0_INT              13
#define XIRQ1_INT              14
#define I2CM_INT               15
#define I2CS_INT               16
#define PWM_INT                17
#define XIRQ2_INT              18
#define XIRQ3_INT              19
